@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 import re
 from itertools import *
 
@@ -8,21 +6,39 @@ from ..h import escapeAttr
 from ..messages import *
 
 
-def parse(lines, numSpacesForIndentation, features=None, opaqueElements=None, blockElements=None):
+def parse(
+    lines,
+    numSpacesForIndentation,
+    features=None,
+    opaqueElements=None,
+    blockElements=None,
+):
     fromStrings = False
-    if any(isinstance(l, str) for l in lines):
+    if any(isinstance(x, str) for x in lines):
         fromStrings = True
-        lines = [Line.Line(-1, l) for l in lines]
+        lines = [Line.Line(-1, x) for x in lines]
     lines = Line.rectify(lines)
-    tokens = tokenizeLines(lines, numSpacesForIndentation, features, opaqueElements=opaqueElements, blockElements=blockElements)
+    tokens = tokenizeLines(
+        lines,
+        numSpacesForIndentation,
+        features,
+        opaqueElements=opaqueElements,
+        blockElements=blockElements,
+    )
     html = parseTokens(tokens, numSpacesForIndentation)
     if fromStrings:
-        return [l.text for l in html]
+        return [x.text for x in html]
     else:
         return html
 
 
-def tokenizeLines(lines, numSpacesForIndentation, features=None, opaqueElements=None, blockElements=None):
+def tokenizeLines(
+    lines,
+    numSpacesForIndentation,
+    features=None,
+    opaqueElements=None,
+    blockElements=None,
+):
     # Turns lines of text into block tokens,
     # which'll be turned into MD blocks later.
     # Every token *must* have 'type', 'raw', and 'prefix' keys.
@@ -32,7 +48,40 @@ def tokenizeLines(lines, numSpacesForIndentation, features=None, opaqueElements=
 
     # Inline elements that are allowed to start a "normal" line of text.
     # Any other element will instead be an HTML line and will close paragraphs, etc.
-    inlineElements = {"a", "em", "strong", "small", "s", "cite", "q", "dfn", "abbr", "data", "time", "code", "var", "samp", "kbd", "sub", "sup", "i", "b", "u", "mark", "ruby", "bdi", "bdo", "span", "br", "wbr", "img", "meter", "progress", "css", "l"}
+    inlineElements = {
+        "a",
+        "em",
+        "strong",
+        "small",
+        "s",
+        "cite",
+        "q",
+        "dfn",
+        "abbr",
+        "data",
+        "time",
+        "code",
+        "var",
+        "samp",
+        "kbd",
+        "sub",
+        "sup",
+        "i",
+        "b",
+        "u",
+        "mark",
+        "ruby",
+        "bdi",
+        "bdo",
+        "span",
+        "br",
+        "wbr",
+        "img",
+        "meter",
+        "progress",
+        "css",
+        "l",
+    }
 
     if blockElements is None:
         blockElements = []
@@ -61,7 +110,7 @@ def tokenizeLines(lines, numSpacesForIndentation, features=None, opaqueElements=
     opaqueElements += ["pre", "xmp", "script", "style"]
     rawElements = "|".join(re.escape(x) for x in opaqueElements)
 
-    for l in lines:
+    for line in lines:
 
         # Three kinds of "raw" elements, which prevent markdown processing inside of them.
         # 1. <pre> and manual opaque elements, which can contain markup and so can nest.
@@ -75,127 +124,156 @@ def tokenizeLines(lines, numSpacesForIndentation, features=None, opaqueElements=
             # Inside at least one raw element that turns off markdown.
             # First see if this line will *end* the raw element.
             endTag = rawStack[-1]
-            if endTag['type'] == "element" and re.search(endTag['tag'], l.text):
+            if endTag["type"] == "element" and re.search(endTag["tag"], line.text):
                 rawStack.pop()
-                tokens.append({'type':'raw', 'prefixlen':float('inf'), 'line':l})
+                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
                 continue
-            elif endTag['type'] == "fenced" and re.match(r"\s*{0}{1}*\s*$".format(endTag['tag'], endTag['tag'][0]), l.text):
+            elif endTag["type"] == "fenced" and re.match(
+                r"\s*{}{}*\s*$".format(endTag["tag"], endTag["tag"][0]), line.text
+            ):
                 rawStack.pop()
-                l.text = "</xmp>"
-                tokens.append({'type':'raw', 'prefixlen':float('inf'), 'line':l})
+                line.text = "</xmp>"
+                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
                 continue
-            elif not endTag['nest']:
+            elif not endTag["nest"]:
                 # Just an internal line, but for the no-nesting elements,
                 # so guaranteed no more work needs to be done.
-                tokens.append({'type':'raw', 'prefixlen':float('inf'), 'line':l})
+                tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
                 continue
 
         # We're either in a nesting raw element or not in a raw element at all,
         # so check if the line starts a new element.
-        match = re.match(r"(\s*)(`{3,}|~{3,})([^`]*)$", l.text)
+        match = re.match(r"(\s*)(`{3,}|~{3,})([^`]*)$", line.text)
         if match:
-            ws,tag,infoString = match.groups()
-            rawStack.append({"type":"fenced", "tag":tag, "nest":False})
+            ws, tag, infoString = match.groups()
+            rawStack.append({"type": "fenced", "tag": tag, "nest": False})
             infoString = infoString.strip()
             if infoString:
                 # For now, I only care about lang
                 lang = infoString.split(" ")[0]
-                classAttr = " class='language-{0}'".format(escapeAttr(lang))
+                classAttr = " class='language-{}'".format(escapeAttr(lang))
             else:
                 classAttr = ""
-            l.text = '{0}<xmp{1}>'.format(ws, classAttr)
-            tokens.append({'type':'raw', 'prefixlen':prefixLen(ws, numSpacesForIndentation), 'line':l})
+            line.text = f"{ws}<xmp{classAttr}>"
+            tokens.append(
+                {
+                    "type": "raw",
+                    "prefixlen": prefixLen(ws, numSpacesForIndentation),
+                    "line": line,
+                }
+            )
             continue
-        match = re.match(r"\s*<({0})[ >]".format(rawElements), l.text)
+        match = re.match(fr"\s*<({rawElements})[ >]", line.text)
         if match:
-            tokens.append({'type':'raw', 'prefixlen':prefixLen(l.text, numSpacesForIndentation), 'line':l})
-            if re.search(r"</({0})>".format(match.group(1)), l.text):
+            tokens.append(
+                {
+                    "type": "raw",
+                    "prefixlen": prefixLen(line.text, numSpacesForIndentation),
+                    "line": line,
+                }
+            )
+            if re.search(r"</({})>".format(match.group(1)), line.text):
                 # Element started and ended on same line, cool, don't need to do anything.
                 pass
             else:
                 nest = match.group(1) not in ["xmp", "script", "style"]
-                rawStack.append({'type':'element', 'tag':"</{0}>".format(match.group(1)), 'nest':nest})
+                rawStack.append(
+                    {
+                        "type": "element",
+                        "tag": "</{}>".format(match.group(1)),
+                        "nest": nest,
+                    }
+                )
             continue
         if rawStack:
-            tokens.append({'type':'raw', 'prefixlen':float('inf'), 'line':l})
+            tokens.append({"type": "raw", "prefixlen": float("inf"), "line": line})
             continue
 
-        line = l.text.strip()
+        lineText = line.text.strip()
 
-        if line == "":
-            token = {'type':'blank',}
+        if lineText == "":
+            token = {
+                "type": "blank",
+            }
         # FIXME: Detect the heading ID from heading lines
-        elif "headings" in features and re.match(r"={3,}\s*$", line):
+        elif "headings" in features and re.match(r"={3,}\s*$", lineText):
             # h1 underline
-            match = re.match(r"={3,}\s$", line)
-            token = {'type':'equals-line'}
-        elif "headings" in features and re.match(r"-{3,}\s*$", line):
+            match = re.match(r"={3,}\s$", lineText)
+            token = {"type": "equals-line"}
+        elif "headings" in features and re.match(r"-{3,}\s*$", lineText):
             # h2 underline
-            match = re.match(r"-{3,}\s*$", line)
-            token = {'type':'dash-line'}
-        elif "headings" in features and re.match(r"(#{1,5})\s+(.+?)(\1\s*\{#[^ }]+\})?\s*$", line):
+            match = re.match(r"-{3,}\s*$", lineText)
+            token = {"type": "dash-line"}
+        elif "headings" in features and re.match(
+            r"(#{1,5})\s+(.+?)(\1\s*\{#[^ }]+\})?\s*$", lineText
+        ):
             # single-line heading
-            match = re.match(r"(#{1,5})\s+(.+?)(\1\s*\{#[^ }]+\})?\s*$", line)
+            match = re.match(r"(#{1,5})\s+(.+?)(\1\s*\{#[^ }]+\})?\s*$", lineText)
             level = len(match.group(1)) + 1
-            token = {'type':'heading', 'text': match.group(2).strip(), 'level': level}
-            match = re.search(r"\{#([^ }]+)\}\s*$", line)
+            token = {"type": "heading", "text": match.group(2).strip(), "level": level}
+            match = re.search(r"\{#([^ }]+)\}\s*$", lineText)
             if match:
-                token['id'] = match.group(1)
-        elif re.match(r"((\*\s*){3,})$|((-\s*){3,})$|((_\s*){3,})$", line):
-            token = {'type':'rule'}
-        elif re.match(r"-?\d+\.\s", line):
-            match = re.match(r"(-?\d+)\.\s+(.*)", line)
-            token = {'type':'numbered', 'text': match.group(2), 'num': int(match.group(1))}
-        elif re.match(r"-?\d+\.$", line):
-            token = {'type':'numbered', 'text': "", 'num': int(line[:-1])}
-        elif re.match(r"[*+-]\s", line):
-            match = re.match(r"[*+-]\s+(.*)", line)
-            token = {'type':'bulleted', 'text': match.group(1)}
-        elif re.match(r"[*+-]$", line):
-            token = {'type':'bulleted', 'text': ""}
-        elif re.match(r":{1,2}\s+", line):
-            match = re.match(r"(:{1,2})\s+(.*)", line)
-            type = 'dt' if len(match.group(1)) == 1 else 'dd'
-            token = {'type':type, 'text': match.group(2)}
-        elif re.match(r":{1,2}$", line):
-            match = re.match(r"(:{1,2})", line)
-            type = 'dt' if len(match.group(1)) == 1 else 'dd'
-            token = {'type':type, 'text': ""}
-        elif re.match(r">", line):
-            match = re.match(r">\s?(.*)", line)
-            token = {'type':'blockquote', 'text':match.group(1)}
-        elif re.match(r"<", line):
-            if re.match(r"<<|<\{", line) or inlineElementStart(line):
-                token = {'type':'text', 'text': line}
+                token["id"] = match.group(1)
+        elif re.match(r"((\*\s*){3,})$|((-\s*){3,})$|((_\s*){3,})$", lineText):
+            token = {"type": "rule"}
+        elif re.match(r"-?\d+\.\s", lineText):
+            match = re.match(r"(-?\d+)\.\s+(.*)", lineText)
+            token = {
+                "type": "numbered",
+                "text": match.group(2),
+                "num": int(match.group(1)),
+            }
+        elif re.match(r"-?\d+\.$", lineText):
+            token = {"type": "numbered", "text": "", "num": int(lineText[:-1])}
+        elif re.match(r"[*+-]\s", lineText):
+            match = re.match(r"[*+-]\s+(.*)", lineText)
+            token = {"type": "bulleted", "text": match.group(1)}
+        elif re.match(r"[*+-]$", lineText):
+            token = {"type": "bulleted", "text": ""}
+        elif re.match(r":{1,2}\s+", lineText):
+            match = re.match(r"(:{1,2})\s+(.*)", lineText)
+            type = "dt" if len(match.group(1)) == 1 else "dd"
+            token = {"type": type, "text": match.group(2)}
+        elif re.match(r":{1,2}$", lineText):
+            match = re.match(r"(:{1,2})", lineText)
+            type = "dt" if len(match.group(1)) == 1 else "dd"
+            token = {"type": type, "text": ""}
+        elif re.match(r">", lineText):
+            match = re.match(r">\s?(.*)", lineText)
+            token = {"type": "blockquote", "text": match.group(1)}
+        elif re.match(r"<", lineText):
+            if re.match(r"<<|<\{", lineText) or inlineElementStart(lineText):
+                token = {"type": "text", "text": lineText}
             else:
-                token = {'type':'htmlblock'}
+                token = {"type": "htmlblock"}
         else:
-            token = {'type':'text', 'text': line}
+            token = {"type": "text", "text": lineText}
 
-        if token['type'] == "blank":
-            token['prefixlen'] = float('inf')
+        if token["type"] == "blank":
+            token["prefixlen"] = float("inf")
         else:
-            token['prefixlen'] = prefixLen(l.text, numSpacesForIndentation)
-        token['line'] = l
+            token["prefixlen"] = prefixLen(line.text, numSpacesForIndentation)
+        token["line"] = line
         tokens.append(token)
 
-    #for token in tokens:
+    # for token in tokens:
     #    print(f"{' ' * (11 - len(token['type']))}{token['type']}: {token['line'].text.rstrip()}")
 
     return tokens
 
 
 def stripComments(lines):
-    '''
+    """
     Eagerly strip comments, because the serializer can't output them right now anyway,
     and they trigger some funky errors.
-    '''
+    """
     output = []
     inComment = False
-    wholeCommentLines = 0
     for line in lines:
         text, inComment = stripCommentsFromLine(line.text, inComment)
-        if (line.text != text and text.strip() == "") or (line.text.strip() == "" and inComment):
+        if (line.text != text and text.strip() == "") or (
+            line.text.strip() == "" and inComment
+        ):
             # First covers the entire line being stripped away by comment-removal.
             # Second covers an empty line that was fully inside a comment.
             # (If a comment started or ended on that line, it wouldn't start out empty.)
@@ -217,7 +295,7 @@ def stripCommentsFromLine(line, inComment=False):
     # Returns true if the comment wasn't closed by the end of the line
     if inComment:
         # The line starts out in a comment.
-        pre,sep,post = line.partition("-->")
+        pre, sep, post = line.partition("-->")
         if sep == "":
             # The entire line is a comment
             return "", True
@@ -226,13 +304,13 @@ def stripCommentsFromLine(line, inComment=False):
             return stripCommentsFromLine(post)
     else:
         # The line starts out as non-comment content.
-        pre,sep,post = line.partition("<!--")
+        pre, sep, post = line.partition("<!--")
         if sep == "":
             # No comments in the line
             return pre, False
         else:
             # Keep the non-comment part, see if there's any more to do
-            res,inComment = stripCommentsFromLine(post, inComment=True)
+            res, inComment = stripCommentsFromLine(post, inComment=True)
             return pre + res, inComment
 
 
@@ -243,7 +321,7 @@ def prefixLen(text, numSpacesForIndentation):
         if text[i] == "\t":
             i += 1
             prefixLen += 1
-        elif text[i:i + numSpacesForIndentation] == " " * numSpacesForIndentation:
+        elif text[i : i + numSpacesForIndentation] == " " * numSpacesForIndentation:
             i += numSpacesForIndentation
             prefixLen += 1
         else:
@@ -252,12 +330,12 @@ def prefixLen(text, numSpacesForIndentation):
 
 
 def stripPrefix(token, numSpacesForIndentation, len):
-    '''Removes len number of prefix groups'''
+    """Removes len number of prefix groups"""
 
-    text = token['line'].text
+    text = token["line"].text
 
     # Don't mess with "infinite" prefix lines
-    if token['prefixlen'] == float('inf'):
+    if token["prefixlen"] == float("inf"):
         return text
 
     # Allow empty lines
@@ -265,18 +343,27 @@ def stripPrefix(token, numSpacesForIndentation, len):
         return text
 
     offset = 0
-    for x in range(len):
+    for _ in range(len):
         if text[offset] == "\t":
             offset += 1
-        elif text[offset:offset + numSpacesForIndentation] == " " * numSpacesForIndentation:
+        elif (
+            text[offset : offset + numSpacesForIndentation]
+            == " " * numSpacesForIndentation
+        ):
             offset += numSpacesForIndentation
         else:
-            die("Line {i} isn't indented enough (needs {0} indent{plural}) to be valid Markdown:\n\"{1}\"", len, text[:-1], plural="" if len == 1 else "s", i=token['line'].i)
+            die(
+                'Line {i} isn\'t indented enough (needs {0} indent{plural}) to be valid Markdown:\n"{1}"',
+                len,
+                text[:-1],
+                plural="" if len == 1 else "s",
+                i=token["line"].i,
+            )
     return text[offset:]
 
 
 def parseTokens(tokens, numSpacesForIndentation):
-    '''
+    """
     Token types:
     eof
     blank
@@ -292,27 +379,30 @@ def parseTokens(tokens, numSpacesForIndentation):
     htmlblock
     blockquote
     raw
-    '''
+    """
     stream = TokenStream(tokens, numSpacesForIndentation)
     lines = []
 
     while True:
         if stream.ended():
             break
-        elif stream.currtype() in ('raw', 'htmlblock'):
+        elif stream.currtype() in ("raw", "htmlblock"):
             lines.append(stream.currline())
             stream.advance()
-        elif stream.currtype() == 'heading':
+        elif stream.currtype() == "heading":
             lines += parseSingleLineHeading(stream)
-        elif stream.currtype() == 'text' and stream.nexttype() in ('equals-line', 'dash-line'):
+        elif stream.currtype() == "text" and stream.nexttype() in (
+            "equals-line",
+            "dash-line",
+        ):
             lines += parseMultiLineHeading(stream)
-        elif stream.currtype() == 'text' and stream.prevtype() == 'blank':
+        elif stream.currtype() == "text" and stream.prevtype() == "blank":
             lines += parseParagraph(stream)
-        elif stream.currtype() == 'rule' or stream.currtype() == 'dash-line':
+        elif stream.currtype() == "rule" or stream.currtype() == "dash-line":
             lines += parseHorizontalRule(stream)
-        elif stream.currtype() == 'bulleted':
+        elif stream.currtype() == "bulleted":
             lines += parseBulleted(stream)
-        elif stream.currtype() == 'numbered':
+        elif stream.currtype() == "numbered":
             lines += parseNumbered(stream, start=stream.currnum())
         elif stream.currtype() in ("dt", "dd"):
             lines += parseDl(stream)
@@ -322,15 +412,17 @@ def parseTokens(tokens, numSpacesForIndentation):
             lines.append(stream.currline())
             stream.advance()
 
-    #for line in lines:
+    # for line in lines:
     #    print "«{0}»".format(line.text.rstrip())
 
     return lines
 
+
 def lineFromStream(stream, text):
     # Shortcut for when you're producing a new line from the currline in the stream,
     # with some modified text.
-    return Line.Line(stream.currline().i,  text)
+    return Line.Line(stream.currline().i, text)
+
 
 # Each parser gets passed the stream
 # and must return the lines it returns.
@@ -340,10 +432,17 @@ def lineFromStream(stream, text):
 
 def parseSingleLineHeading(stream):
     if "id" in stream.curr():
-        idattr = " id='{0}'".format(stream.currid())
+        idattr = f" id='{stream.currid()}'"
     else:
         idattr = ""
-    lines = [lineFromStream(stream, "<h{level}{idattr} line-number={i}>{text}</h{level}>\n".format(idattr=idattr, i=stream.currline().i, **stream.curr()))]
+    lines = [
+        lineFromStream(
+            stream,
+            "<h{level}{idattr} line-number={i}>{text}</h{level}>\n".format(
+                idattr=idattr, i=stream.currline().i, **stream.curr()
+            ),
+        )
+    ]
     stream.advance()
     return lines
 
@@ -354,21 +453,37 @@ def parseMultiLineHeading(stream):
     elif stream.nexttype() == "dash-line":
         level = 3
     else:
-        die("Markdown parser error: tried to parse a multiline heading from:\n{0}{1}{2}", stream.prevraw(), stream.currraw(), stream.nextraw())
+        die(
+            "Markdown parser error: tried to parse a multiline heading from:\n{0}{1}{2}",
+            stream.prevraw(),
+            stream.currraw(),
+            stream.nextraw(),
+        )
     match = re.search(r"(.*?)\s*\{\s*#([^ }]+)\s*\}\s*$", stream.currtext())
     if match:
         text = match.group(1)
-        idattr = "id='{0}'".format(match.group(2))
+        idattr = "id='{}'".format(match.group(2))
     else:
         text = stream.currtext()
         idattr = ""
-    lines = [lineFromStream(stream,  "<h{level} {idattr} line-number={i}>{htext}</h{level}>\n".format(idattr=idattr, level=level, htext=text, i=stream.currline().i, **stream.curr()))]
+    lines = [
+        lineFromStream(
+            stream,
+            "<h{level} {idattr} line-number={i}>{htext}</h{level}>\n".format(
+                idattr=idattr,
+                level=level,
+                htext=text,
+                i=stream.currline().i,
+                **stream.curr(),
+            ),
+        )
+    ]
     stream.advance(2)
     return lines
 
 
 def parseHorizontalRule(stream):
-    lines = [lineFromStream(stream,  "<hr>\n")]
+    lines = [lineFromStream(stream, "<hr>\n")]
     stream.advance()
     return lines
 
@@ -383,27 +498,34 @@ def parseParagraph(stream):
         matchNote = re.match(r"(note[:,])(\s*)(.*)", line, re.I)
         if matchNote:
             line = matchNote.group(3)
-            p += "<span>{note}</span>{ws}".format(note=matchNote.group(1), ws=matchNote.group(2))
+            p += "<span>{note}</span>{ws}".format(
+                note=matchNote.group(1), ws=matchNote.group(2)
+            )
     elif line.lower().startswith("issue:"):
         line = line[6:]
-        p = "<p line-number={0} class='replace-with-issue-class'>".format(i)
+        p = f"<p line-number={i} class='replace-with-issue-class'>"
     elif line.lower().startswith("assert:"):
-        p = "<p line-number={0} class='replace-with-assertion-class'>".format(i)
+        p = f"<p line-number={i} class='replace-with-assertion-class'>"
     elif line.lower().startswith("advisement:"):
         line = line[11:]
-        p = "<p line-number={0}><strong class='replace-with-advisement-class'>".format(i)
+        p = f"<p line-number={i}><strong class='replace-with-advisement-class'>"
         endTag = "</strong></p>"
     else:
         match = re.match(r"issue\(([^)]+)\):(.*)", line, re.I)
         if match:
             line = match.group(2)
-            p = "<p line-number={0} data-remote-issue-id='{1}' class='replace-with-issue-class'>".format(i, match.group(1))
+            p = "<p line-number={} data-remote-issue-id='{}' class='replace-with-issue-class'>".format(
+                i, match.group(1)
+            )
         else:
-            p = "<p line-number={0}>".format(i)
-    lines = [lineFromStream(stream, "{0}{1}\n".format(p, line))]
+            p = f"<p line-number={i}>"
+    lines = [lineFromStream(stream, f"{p}{line}\n")]
     while True:
         stream.advance()
-        if stream.currtype() not in ["text"] or stream.currprefixlen() < initialPrefixLen:
+        if (
+            stream.currtype() not in ["text"]
+            or stream.currprefixlen() < initialPrefixLen
+        ):
             lines[-1].text = lines[-1].text.rstrip() + endTag + "\n"
             return lines
         lines.append(stream.currline())
@@ -419,37 +541,50 @@ def parseBulleted(stream):
         # Remove the bulleted part from the line
         firstLine = stream.currtext() + "\n"
         i = stream.currline().i
-        lines = [lineFromStream(stream,  firstLine)]
+        lines = [lineFromStream(stream, firstLine)]
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
-            if stream.currtype() == 'bulleted' and stream.currprefixlen() == prefixLen:
+            if stream.currtype() == "bulleted" and stream.currprefixlen() == prefixLen:
                 return lines, i
             if stream.currprefixlen() < prefixLen:
                 return lines, i
-            if stream.currtype() == 'blank' and stream.nexttype() != 'bulleted' and stream.nextprefixlen() <= prefixLen:
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() != "bulleted"
+                and stream.nextprefixlen() <= prefixLen
+            ):
                 return lines, i
-            if stream.currtype() == 'eof':
+            if stream.currtype() == "eof":
                 return lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(lineFromStream(stream,  stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1)))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                )
+            )
 
     def getItems(stream):
         while True:
             # The conditions that indicate we're past the end of the list itself
-            if stream.currtype() == 'eof':
+            if stream.currtype() == "eof":
                 return
-            if stream.currtype() == 'blank' and stream.nexttype() != 'bulleted' and stream.nextprefixlen() <= prefixLen:
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() != "bulleted"
+                and stream.nextprefixlen() <= prefixLen
+            ):
                 return
             if stream.currprefixlen() < prefixLen:
                 return
-            if stream.currtype() == 'blank':
+            if stream.currtype() == "blank":
                 stream.advance()
             yield parseItem(stream)
 
-    lines = [Line.Line(-1,  "<ul data-md line-number={0}>".format(ul_i))]
+    lines = [Line.Line(-1, f"<ul data-md line-number={ul_i}>")]
     for li_lines, i in getItems(stream):
-        lines.append(Line.Line(-1, "<li data-md line-number={0}>".format(i)))
+        lines.append(Line.Line(-1, f"<li data-md line-number={i}>"))
         lines.extend(parse(li_lines, numSpacesForIndentation))
         lines.append(Line.Line(-1, "</li>"))
     lines.append(Line.Line(-1, "</ul>"))
@@ -470,36 +605,49 @@ def parseNumbered(stream, start=1):
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
-            if stream.currtype() == 'numbered' and stream.currprefixlen() == prefixLen:
-                return lines,i
+            if stream.currtype() == "numbered" and stream.currprefixlen() == prefixLen:
+                return lines, i
             if stream.currprefixlen() < prefixLen:
-                return lines,i
-            if stream.currtype() == 'blank' and stream.nexttype() != 'numbered' and stream.nextprefixlen() <= prefixLen:
-                return lines,i
-            if stream.currtype() == 'eof':
-                return lines,i
+                return lines, i
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() != "numbered"
+                and stream.nextprefixlen() <= prefixLen
+            ):
+                return lines, i
+            if stream.currtype() == "eof":
+                return lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(lineFromStream(stream, stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1)))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                )
+            )
 
     def getItems(stream):
         while True:
             # The conditions that indicate we're past the end of the list itself
-            if stream.currtype() == 'eof':
+            if stream.currtype() == "eof":
                 return
-            if stream.currtype() == 'blank' and stream.nexttype() != 'numbered' and stream.nextprefixlen() <= prefixLen:
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() != "numbered"
+                and stream.nextprefixlen() <= prefixLen
+            ):
                 return
             if stream.currprefixlen() < prefixLen:
                 return
-            if stream.currtype() == 'blank':
+            if stream.currtype() == "blank":
                 stream.advance()
             yield parseItem(stream)
 
     if start == 1:
-        lines = [Line.Line(-1, "<ol data-md line-number={0}>".format(ol_i))]
+        lines = [Line.Line(-1, f"<ol data-md line-number={ol_i}>")]
     else:
-        lines = [Line.Line(-1, "<ol data-md start='{0}' line-number={1}>".format(start, ol_i))]
-    for li_lines,i in getItems(stream):
-        lines.append(Line.Line(-1, "<li data-md line-number={0}>".format(i)))
+        lines = [Line.Line(-1, f"<ol data-md start='{start}' line-number={ol_i}>")]
+    for li_lines, i in getItems(stream):
+        lines.append(Line.Line(-1, f"<li data-md line-number={i}>"))
         lines.extend(parse(li_lines, numSpacesForIndentation))
         lines.append(Line.Line(-1, "</li>"))
     lines.append(Line.Line(-1, "</ol>"))
@@ -520,35 +668,51 @@ def parseDl(stream):
         while True:
             stream.advance()
             # All the conditions that indicate we're *past* the end of the item.
-            if stream.currtype() in ('dt', 'dd') and stream.currprefixlen() == prefixLen:
+            if (
+                stream.currtype() in ("dt", "dd")
+                and stream.currprefixlen() == prefixLen
+            ):
                 return type, lines, i
             if stream.currprefixlen() < prefixLen:
                 return type, lines, i
-            if stream.currtype() == 'blank' and stream.nexttype() not in ('dt', 'dd') and stream.nextprefixlen() <= prefixLen:
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() not in ("dt", "dd")
+                and stream.nextprefixlen() <= prefixLen
+            ):
                 return type, lines, i
-            if stream.currtype() == 'eof':
+            if stream.currtype() == "eof":
                 return type, lines, i
             # Remove the prefix from each line before adding it.
-            lines.append(lineFromStream(stream, stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1)))
+            lines.append(
+                lineFromStream(
+                    stream,
+                    stripPrefix(stream.curr(), numSpacesForIndentation, prefixLen + 1),
+                )
+            )
 
     def getItems(stream):
         while True:
             # The conditions that indicate we're past the end of the list itself
-            if stream.currtype() == 'eof':
+            if stream.currtype() == "eof":
                 return
-            if stream.currtype() == 'blank' and stream.nexttype() not in ('dt', 'dd') and stream.nextprefixlen() <= prefixLen:
+            if (
+                stream.currtype() == "blank"
+                and stream.nexttype() not in ("dt", "dd")
+                and stream.nextprefixlen() <= prefixLen
+            ):
                 return
             if stream.currprefixlen() < prefixLen:
                 return
-            if stream.currtype() == 'blank':
+            if stream.currtype() == "blank":
                 stream.advance()
             yield parseItem(stream)
 
-    lines = [Line.Line(-1, "<dl data-md line-number={0}>".format(dl_i))]
+    lines = [Line.Line(-1, f"<dl data-md line-number={dl_i}>")]
     for type, di_lines, i in getItems(stream):
-        lines.append(Line.Line(-1, "<{0} data-md line-number={1}>".format(type, i)))
+        lines.append(Line.Line(-1, f"<{type} data-md line-number={i}>"))
         lines.extend(parse(di_lines, numSpacesForIndentation))
-        lines.append(Line.Line(-1, "</{0}>".format(type)))
+        lines.append(Line.Line(-1, f"</{type}>"))
     lines.append(Line.Line(-1, "</dl>"))
     return lines
 
@@ -556,20 +720,30 @@ def parseDl(stream):
 def parseBlockquote(stream):
     prefixLen = stream.currprefixlen()
     i = stream.currline().i
-    lines = [lineFromStream(stream, stream.currtext()+"\n")]
+    lines = [lineFromStream(stream, stream.currtext() + "\n")]
     while True:
         stream.advance()
         if stream.currprefixlen() < prefixLen:
             break
         if stream.currtype() in ["blockquote", "text"]:
-            lines.append(lineFromStream(stream, stream.currtext()+"\n"))
+            lines.append(lineFromStream(stream, stream.currtext() + "\n"))
         else:
             break
-    return [Line.Line(-1, "<blockquote line-number={0}>\n".format(i))] + parse(lines, stream.numSpacesForIndentation) + [Line.Line(-1, "</blockquote>\n")]
+    return (
+        [Line.Line(-1, f"<blockquote line-number={i}>\n")]
+        + parse(lines, stream.numSpacesForIndentation)
+        + [Line.Line(-1, "</blockquote>\n")]
+    )
 
 
 class TokenStream:
-    def __init__(self, tokens, numSpacesForIndentation, before={'type':'blank','prefixlen':0}, after={'type':'eof','prefixlen':0}):
+    def __init__(
+        self,
+        tokens,
+        numSpacesForIndentation,
+        before={"type": "blank", "prefixlen": 0},
+        after={"type": "eof", "prefixlen": 0},
+    ):
         self.tokens = tokens
         self.i = 0
         self.numSpacesForIndentation = numSpacesForIndentation
@@ -617,7 +791,7 @@ class TokenStream:
                     tok = self.curr()
                 if attrName in tok:
                     return tok[attrName]
-                else:
-                    raise AttributeError(attrName)
-                    return tok['raw']
+
+                raise AttributeError(attrName)
+
             return _missing
